package at.kugel.tool.buildtray.action;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import at.kugel.tool.buildtray.config.Config;

/**
 * Strategy for String related HTTP methods.
 */
public class StringIOHelper {

   private final Config config;

   public StringIOHelper(Config pConfig) {
      config = pConfig;
   }

   public String readPage() throws IOException {
      List<Map.Entry<String, String>> headers = new ArrayList<Map.Entry<String, String>>();
      setProxySettings();
      setSSLSettings();
      setBasicAuth(headers);
      return readNet(new URL(config.getServerlUrl()), headers);
   }

   private void setProxySettings() {
      if (config.useProxy()) {
         System.setProperty("proxySet", Boolean.TRUE.toString());
         System.setProperty("proxyHost", config.getProxyHost());
         System.setProperty("proxyPort", Integer.toString(config.getProxyPort()));
      } else {
         System.setProperty("proxySet", Boolean.FALSE.toString());
         System.getProperties().remove("proxyHost");
         System.getProperties().remove("proxyPort");
      }
   }

   private void setSSLSettings() {
      if (config.getServerlUrl().startsWith("https://")) {
         if (config.useTrustStore()) {
            System.setProperty("javax.net.ssl.trustStore", config.getTrustStore());
            System.setProperty("javax.net.ssl.trustStorePassword", config.getTrustStorePassword());
         } else {
            System.getProperties().remove("javax.net.ssl.trustStore");
            System.getProperties().remove("javax.net.ssl.trustStorePassword");
            SSLHelper.disableHTTPSChecks();
         }
      }
   }

   private void setBasicAuth(List<Map.Entry<String, String>> headers) {
      if (config.useAuth()) {
         String userPassword = config.getServerUsername() + ':' + config.getServerPassword();
         String encoding = new sun.misc.BASE64Encoder().encode(userPassword.getBytes());
         headers.add(new StringIOHelper.HeaderEntry("Authorization", "Basic " + encoding));
      }
   }

   /**
    * Load a textfile from the net into a string.
    * @param pUrl the url to load.
    * @param headers the headers to send and on exit the headers which were sent.
    * @return a string of the file loaded.
    * @throws IOException if the string can not be read.
    */
   private static String readNet(URL pUrl, List<Map.Entry<String, String>> headers) throws IOException {
      final URLConnection con = pUrl.openConnection();
      putTableIntoHeaders(con, headers);
      con.connect();
      putHeadersIntoTable(con, headers);

      int size = con.getContentLength();
      boolean sizeGiven = true;
      if (size < 0) {
         sizeGiven = false;
         // guess size
         size = 256 * 1024;
      }
      final byte[] data = new byte[size];

      InputStream ins = null;
      try {
         ins = con.getInputStream();
         // comes in parts - read parts
         int pos = 0;
         int res = 1;
         while (pos < size && res > 0) {
            res = ins.read(data, pos, size - pos);
            pos += res;
         }

         if (sizeGiven && pos < size) {
            throw new IOException("short file: length=" + pos + " != content-length=" + size);
         } else if (!sizeGiven && pos == size) {
            throw new IOException("possibly long file: length=" + pos + " != content-length=unknown");
         } else if (!sizeGiven) {
            pos++;
         }

         final String encoding = con.getContentEncoding();
         if (encoding == null) {
            return new String(data, 0, pos);
         }

         return new String(data, 0, pos, encoding);

      } finally {
         if (ins != null) {
            ins.close();
         }
      }

   }

   private static void putTableIntoHeaders(URLConnection con, List<Map.Entry<String, String>> headers) {
      if (headers != null) {
         for (Map.Entry<String, String> elem : headers) {
            if (elem.getKey().equals("")) {
               // first header...
               continue;
            }
            con.setRequestProperty(elem.getKey(), elem.getValue());
         }
         headers.clear();
      }
   }

   private static void putHeadersIntoTable(URLConnection con, List<Map.Entry<String, String>> headers) {
      if (headers != null) {
         headers.clear();

         int i = 0;
         String val = con.getHeaderField(i);
         while (val != null) {
            String key = con.getHeaderFieldKey(i);
            if (key == null) {
               // first header...
               key = "";
            }

            headers.add(new HeaderEntry(key, val));

            val = con.getHeaderField(++i);
         }
      }
   }

   private static class HeaderEntry implements Map.Entry<String, String> {

      private final String key;
      private String value;

      public HeaderEntry(String pKey, String pValue) {
         key = pKey;
         value = pValue;
      }

      @Override
      public String getKey() {
         return key;
      }

      @Override
      public String getValue() {
         return value;
      }

      @Override
      public String setValue(String pValue) {
         String oldValue = value;
         value = pValue;
         return oldValue;
      }

   }
}
